# enumalg.gi
#############################################################################
##
#O  NumberOfImmediateDescendantsOfCanoForm( <C>, <s> ) . . . . . . . . . . . . . . . 
#O  NumberOfImmediateDescendantsOfCanoForm( <C>, <slist> ) . . . . . . . . . . . . . . . 
#O  NumberOfImmediateDescendantsOfCanoForm( <C> ) . . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="NumberOfImmediateDescendantsOfCanoForm">
##  <ManSection>
##  <Oper Name="NumberOfImmediateDescendantsOfCanoForm" Arg="C, s" Label="for a single stepsize"/>
##
##  <Description>
##  Given a canonical form <A>C</A> this returns the number of immediate descendants
##  of <A>C</A> with stepsize <A>s</A>.
##  </Description>
##  </ManSection>
##  <ManSection>
##  <Oper Name="NumberOfImmediateDescendantsOfCanoForm" Arg="C, slist" Label="for a range of stepsizes"/>
##
##  <Description>
##  Given a canonical form <A>C</A> this returns the number of immediate descendants
##  of <A>C</A> with stepsizes in <A>slist</A>.
##  </Description>
##  </ManSection>
##  <ManSection>
##  <Oper Name="NumberOfImmediateDescendantsOfCanoForm" Arg="C" Label="for all stepsizes"/>
##  
##  <Description>
##  Given a canonical form <A>C</A> this returns the number of immediate descendants
##  of <A>C</A> of all stepsizes.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NumberOfImmediateDescendantsOfCanoForm", [IsRecord, IsPosInt] );
DeclareOperation( "NumberOfImmediateDescendantsOfCanoForm", [IsRecord, IsList] );
DeclareOperation( "NumberOfImmediateDescendantsOfCanoForm", [IsRecord] );
DeclareOperation( "NumberOfImmediateDescendantsOfNAAlg", [IsAlgebra and IsFiniteDimensional, IsPosInt] );
DeclareOperation( "NumberOfImmediateDescendantsOfNAAlg", [IsAlgebra and IsFiniteDimensional, IsList] );
DeclareOperation( "NumberOfImmediateDescendantsOfNAAlg", [IsAlgebra and IsFiniteDimensional] );

#############################################################################
##
#O  NumberOfNAAlgsByDimAndGens( <F>, <d>, <n> ) . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="NumberOfNAAlgsByDimAndGens">
##  <ManSection>
##  <Oper Name="NumberOfNAAlgsByDimAndGens" Arg="F, d, n"/>
##  
##  <Description>
##  Given a finite field <A>F</A>, a generator number <A>d</A> and a dimension <A>n</A>, 
##  this function returns the number of <A>d</A>-generator nilpotent associative <A>F</A>-algebras 
##  of dimension <A>n</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NumberOfNAAlgsByDimAndGens", [IsField and IsFinite, IsPosInt, IsPosInt] );

#############################################################################
##
#O  NumberOfNAAlgsByDim( <F>, <n> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="NumberOfNAAlgsByDim">
##  <ManSection>
##  <Oper Name="NumberOfNAAlgsByDim" Arg="F, n"/>
##  
##  <Description>
##  Given a finite field <A>F</A> and a dimension <A>n</A>, this function returns the
##  number of nilpotent associative <A>F</A>-algebras of dimension <A>n</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NumberOfNAAlgsByDim", [IsField and IsFinite, IsPosInt] );


# genalg.gi
#############################################################################
##
#O  NAAlgByCanoForm( <C> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="NAAlgByCanoForm">
##  <ManSection>
##  <Oper Name="NAAlgByCanoForm" Arg="C"/>
##  
##  <Description>
##  Converts a canonical form <A>C</A> to a nilpotent associative algebra.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NAAlgByCanoForm", [IsRecord] );

#############################################################################
##
#O  CanoFormByNAAlg( <A> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="CanoFormByNAAlg">
##  <ManSection>
##  <Oper Name="CanoFormByNAAlg" Arg="A"/>
##  
##  <Description>
##  Converts a nilpotent associative algebra <A>A</A> to a canonical form.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "CanoFormByNAAlg", [IsAlgebra and IsFiniteDimensional] );

#############################################################################
##
#O  ImmediateDescendantsOfCanoForm( <C>, <s> ) . . . . . . . . . . . . . . . . . . 
#O  ImmediateDescendantsOfCanoForm( <C>, <slist> ) . . . . . . . . . . . . . . . . . 
#O  ImmediateDescendantsOfCanoForm( <C> ) . . . . . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="ImmediateDescendantsOfCanoForm">
##  <ManSection>
##  <Oper Name="ImmediateDescendantsOfCanoForm" Arg="C, s" Label="for a single stepsize"/>
##  <Description>
##  Given a canonical form <A>C</A> this function returns the immediate descendants
##  of <A>C</A> with stepsize <A>s</A>.
##
##  </Description>
##  </ManSection>
##  <ManSection>
##  <Oper Name="ImmediateDescendantsOfCanoForm" Arg="C, slist" Label="for a range of stepsizes"/>
##
##  <Description>
##  Given a canonical form <A>C</A> this function returns the immediate descendants
##  of <A>C</A> with stepsizes in <A>slist</A>.
##  </Description>
##  </ManSection>
##  <ManSection>
##  <Oper Name="ImmediateDescendantsOfCanoForm" Arg="C" Label="for all stepsizes"/>
##  
##  <Description>
##  Given a canonical form <A>C</A> this function returns the immediate descendants
##  of <A>C</A> of all stepsizes.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "ImmediateDescendantsOfCanoForm", [IsRecord, IsPosInt] );
DeclareOperation( "ImmediateDescendantsOfCanoForm", [IsRecord, IsList] );
DeclareOperation( "ImmediateDescendantsOfCanoForm", [IsRecord] );
DeclareOperation( "ImmediateDescendantsOfNAAlg", [IsAlgebra and IsFiniteDimensional, IsPosInt] );
DeclareOperation( "ImmediateDescendantsOfNAAlg", [IsAlgebra and IsFiniteDimensional, IsList] );
DeclareOperation( "ImmediateDescendantsOfNAAlg", [IsAlgebra and IsFiniteDimensional] );

#############################################################################
##
#P  IsCapableCanoForm( <C> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="IsCapableCanoForm">
##  <ManSection>
##  <Prop Name="IsCapableCanoForm" Arg="C"/>
##  
##  <Description>
##  This returns whether the given canonical form <A>C</A> is capable, i.e. whether 
##  it has descendants.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareProperty( "IsCapableCanoForm", IsRecord );
DeclareProperty( "IsCapableNAAlg", IsAlgebra and IsFiniteDimensional );

#############################################################################
##
#A  CoclassOfCanoForm( <C> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="CoclassOfCanoForm">
##  <ManSection>
##  <Attr Name="CoclassOfCanoForm" Arg="C"/>
##  
##  <Description>
##  This returns the coclass of the given canonical form <A>C</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareAttribute( "CoclassOfCanoForm", IsRecord );
DeclareAttribute( "CoclassOfNAAlg", IsAlgebra and IsFiniteDimensional );

#############################################################################
##
#A  IsCoclassSettledCanoForm( <C> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="IsCoclassSettledCanoForm">
##  <ManSection>
##  <Attr Name="IsCoclassSettledCanoForm" Arg="C"/>
##  
##  <Description>
##  Determines whether the given canonical form <A>C</A> is coclass settled.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareAttribute( "IsCoclassSettledCanoForm", IsRecord );
DeclareAttribute( "IsCoclassSettledNAAlg", IsAlgebra and IsFiniteDimensional );

#############################################################################
##
#O  RootsOfCoclassGraph( <F>, <r> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="RootsOfCoclassGraph">
##  <ManSection>
##  <Oper Name="RootsOfCoclassGraph" Arg="F, r"/>
##  
##  <Description>
##  Given a finite field <A>F</A>, a coclass <A>r</A> this function calculates the roots  
##  of the maximal descendant trees of the associated coclass graph.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "RootsOfCoclassGraph", [IsField and IsFinite, IsInt] );

#############################################################################
##
#O  RootsOfCoclassGraphByRank( <F>, <r>, <d> ) . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="RootsOfCoclassGraphByRank">
##  <ManSection>
##  <Oper Name="RootsOfCoclassGraphByRank" Arg="F, r, d"/>
##  
##  <Description>
##  Given a finite field <A>F</A>, a coclass <A>r</A> and a generator number <A>d</A>
##	this function calculates the roots with the given number of generators of the maximal 
##  descendant trees of the associated coclass graph.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "RootsOfCoclassGraphByRank", [IsField and IsFinite, IsInt, IsInt] );

#############################################################################
##
#O  DescendantTreeOfCanoForm( <C>, <l> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="DescendantTreeOfCanoForm">
##  <ManSection>
##  <Oper Name="DescendantTreeOfCanoForm" Arg="C, l"/>
##  
##  <Description>
##  Given a canonical form <A>C</A> for a nilpotent associative <M>F</M>-algebra,
##  this iteratedly calculates <A>l</A> levels of descendants of the same coclass
##  as <A>C</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "DescendantTreeOfCanoForm", [IsRecord, IsPosInt] );
DeclareOperation( "DescendantTreeOfNAAlg", [IsAlgebra and IsFiniteDimensional, IsPosInt] );

#############################################################################
##
#O  CanoFormsByDimAndGens( <F>, <d>, <n> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="CanoFormsByDimAndGens">
##  <ManSection>
##  <Oper Name="CanoFormsByDimAndGens" Arg="F, d, n"/>
##  
##  <Description>
##  Given a finite field <A>F</A>, a generator number <A>d</A> and a dimension <A>n</A>, 
##  this function calculates canonical forms for the <A>d</A>-generator nilpotent associative 
##  <A>F</A>-algebras of dimension <A>n</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "CanoFormsByDimAndGens", [IsField and IsFinite, IsPosInt, IsPosInt] );

#############################################################################
##
#O  NAAlgsByDimAndGens( <F>, <d>, <n> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="NAAlgsByDimAndGens">
##  <ManSection>
##  <Oper Name="NAAlgsByDimAndGens" Arg="F, d, n"/>
##  
##  <Description>
##  Given a finite field <A>F</A>, a generator number <A>d</A> and a dimension <A>n</A>, 
##  this function calculates the <A>d</A>-generator nilpotent associative <A>F</A>-algebras 
##  of dimension <A>n</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NAAlgsByDimAndGens", [IsField and IsFinite, IsPosInt, IsPosInt] );

#############################################################################
##
#O  CanoFormsByDim( <F>, <n> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="CanoFormsByDim">
##  <ManSection>
##  <Oper Name="CanoFormsByDim" Arg="F, n"/>
##  
##  <Description>
##  Given a finite field <A>F</A> and a dimension <A>n</A>, this function
##  calculates canonical forms for the nilpotent associative <A>F</A>-algebras
##  of dimension <A>n</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "CanoFormsByDim", [IsField and IsFinite, IsPosInt] );

#############################################################################
##
#O  NAAlgsByDim( <F>, <n> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="NAAlgsByDim">
##  <ManSection>
##  <Oper Name="NAAlgsByDim" Arg="F, n"/>
##  
##  <Description>
##  Given a finite field <A>F</A> and a dimension <A>n</A>, this function
##  calculates the nilpotent associative <A>F</A>-algebras of dimension <A>n</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "NAAlgsByDim", [IsField and IsFinite, IsPosInt] );


# Load XGAP stuff only if xgap is active
if TestPackageAvailability("xgap") <> fail then
# plot.gi	
#############################################################################
##
#O  PlotCoclassGraph( <F>, <r>, <d>, <P> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="PlotCoclassGraph">
##  <ManSection>
##  <Oper Name="PlotCoclassGraph" Arg="F, r, d, P"/>
##  
##  <Description>
##	Given a finite field <A>F</A> and a coclass <A>r</A>, this plots the associated
##	coclass graph up to dimension <A>d</A> into the GraphicPoset <A>P</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "PlotCoclassGraph", [IsField and IsFinite, IsInt, IsPosInt, IsGraphicSheet] );

#############################################################################
##
#O  PlotDescendantTreeOfCanoForm( <C>, <d>, <P> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="PlotDescendantTreeOfCanoForm">
##  <ManSection>
##  <Oper Name="PlotDescendantTreeOfCanoForm" Arg="C, d, P"/>
##  
##  <Description>
##	Given a canonical form <A>C</A>, this plots the descendant tree of canonical 
##	forms of the same coclass as <A>C</A> up to dimension <A>d</A> into the
##	GraphicPoset <A>P</A>.  
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "PlotDescendantTreeOfCanoForm", [IsRecord, IsPosInt, IsGraphicSheet] );

#############################################################################
##
#O  GetSelectedCanoForms( <P> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="GetSelectedCanoForms">
##  <ManSection>
##  <Oper Name="GetSelectedCanoForms" Arg="P"/>
##  
##  <Description>
##	Returns a list of canonical forms corresponding to the selected vertices
##	in the GraphicPoset <A>P</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "GetSelectedCanoForms", [IsGraphicSheet] );

#############################################################################
##
#O  GetSelectedNAAlgs( <P> ) . . . . . . . . . . . . . . . . . 
##
##  <#GAPDoc Label="GetSelectedNAAlgs">
##  <ManSection>
##  <Oper Name="GetSelectedNAAlgs" Arg="P"/>
##  
##  <Description>
##	Returns a list of nilpotent associative algebras corresponding to the selected 
##	vertices in the GraphicPoset <A>P</A>.
##  </Description>
##  </ManSection>
##  <#/GAPDoc>
DeclareOperation( "GetSelectedNAAlgs", [IsGraphicSheet] );
fi;
