TensorAction := function( mat, f )
    return KroneckerProduct( mat, mat );
end;

MyJordanBlockLengths := function( g, F )
    local chr, min, fc, fm, ns, V, W, U, i, f, e, h, k, v, t, R;

	R := PolynomialRing(F, ["x_1"]);

    # char poly 
    chr := CharacteristicPolynomial(F, F, g);
    fc := Collected(Factors(R, chr));

    # min poly 
    min := MinimalPolynomial(F, g);
    fm := Factors(R, min);

    # set up
    ns := List( fc, x -> List( [1..x[2]], y -> 0 ) );
    V := g^0;

    # loop 
    for i in [1..Length(fc)] do

        # determine V_i
        f := fc[i][1];
        e := Length(Filtered(fm, x -> x = f)); 
        h := Value( f^e, g );
        W := TriangulizedNullspaceMat(h);
        k := InducedActionFactor( [g], W, [] )[1];

        # split cyclic
        while not IsBool(k) do 
            ns[i][e] := ns[i][e] + 1;
            if Length(k) = e * Degree(f) then 
                k := false;
            else
                h := Value( f^(e-1), k );
                v := First( k^0, x -> x*h <> 0*x );
                W := SpinUpCyclic( v, k, e*Degree(f) );
                U := BaseSteinitzVectors(k^0, W).factorspace;
                k := InducedActionFactor( [k], U, W )[1];
                t := Collected(Factors(R, MinimalPolynomial( F, k )));
                e := t[1][2];
            fi;
        od;
    od;
    return rec( blks := ns, degs := List(fc, x -> Degree(x[1])));
end;

MyCountOrbitsGL := function( n, p, k, Action )
    local G, cls, jor, len, fix, cl, g, h, J, t, j, l;

    # set up
    G := GL(n,p);
    cls := ConjugacyClasses(G);

    # catch input
    if IsBool(k) then l := n*(n-1)/2-1; fi;

    # infos
    jor := [];
    len := [];
    fix := [];

    for cl in cls do

        # get g and action
        g := Representative(cl);
        h := Action(g, GF(p));
        Info(InfoAutGrp, 1, "start class of order ", Order(g));

        # compute jordan blocks n_{ij}
        J := MyJordanBlockLengths(h, GF(p));

        # check if known
        j := Position( jor, J );
        if IsBool( j ) then 
            Info( InfoAutGrp, 2, "  Degree: ",J.degs," -- Jordan ", J.blks);
            if IsBool(k) then 
                t := List([1..l], x -> FixedPointsByDecom(J.blks,J.degs,p,x));
            else
                t := FixedPointsByDecom( J.blks, J.degs, p, k );
            fi;

            Add( jor, J );
            Add( len, Size(cl) );
            Add( fix, t );
            Info( InfoAutGrp, 2, "  class yields ",t);
        else
            len[j] := len[j] + Size(cl);
        fi;
    od;

    return Sum( List( [1..Length(jor)], i -> len[i]*fix[i] ) ) / Size(G);
end;


NumberOfClass2AssociativeAlgebras := function(d, p, k)
    return MyCountOrbitsGL( d, p, k, TensorAction );
end;


NumberOfClass2AssociativeAlgebrasByDim := function(d, p)
local r, i;

    if d = 1 then
        return 0;
    fi;

    r := 0;
	
    for i in [1..d-1] do
        r := r + NumberOfClass2AssociativeAlgebras(i, p, d-i);
    od;

    return r;
end;


NumberOfFixedPointsOfOnBases := function( bases, mat )
	if Length(bases) = 1 then
		return 1;
	fi;

	return Size(Filtered(bases, x -> TriangulizedMat(x*mat) = TriangulizedMat(x)));
end;


NumberOfQuotientsByAllowables := function( C, G, s )
    local d, I, S, iter, all, i, r, M, classes, sumfix, g;

    d := C.mul;
    if s < 0 or d < s then return 0; fi;
	#if d = s then return 1; fi;

    # get all subspaces
    S := Subspaces( C.fld^d, d-s);
    Info( InfoModIsom, 1, "   determined ",Size(S)," subspaces");
    iter := Iterator(S);

    # filter for supplements
    I := IdentityMat(d,C.fld){[1..C.nuc]};
    all := [];

    for i in [1..Size(S)] do
       	s := BasisVectors(Basis(NextIterator(iter)));
        if C.nuc < C.mul then 
            r := RankMat(Concatenation(I, s));
            if r = d then Add( all, s ); fi;
        else
            Add(all, s);
        fi;
    od;
    Info( InfoModIsom, 1, "   reduced to ",Length(all)," supplements");

    # determine orbits
    M := Group( Concatenation(List(G.agAutos, x -> x[2]),
                              List(G.glAutos, x -> x[2])),
                G.one[2] );

	classes := ConjugacyClasses(M);
	sumfix := 0;

	for g in classes do
		sumfix := sumfix + Size(g) * NumberOfFixedPointsOfOnBases(all, Representative(g));
	od;

	return sumfix / Size(M);
end;


# Calculates the number of immediate descendants of the given cano form of stepsize s
InstallMethod( NumberOfImmediateDescendantsOfCanoForm, [IsRecord, IsPosInt], function( A, s )
    local T, G, C, all, i;

    # catch some arguments from  
    T := StructuralCopy(A.cano);
    G := StructuralCopy(A.auto);

    # compute cover
    C := CoveringTable( T );
    Info( InfoModIsom, 1, "   dim(M) = ",C.mul, " and dim(N) = ",C.nuc );
    if C.nuc = 0 then return 0; fi;
    if s = 0 then s := C.nuc; fi;

    # lift autos
    InduceAutosToMult( G, C, T );
    Info( InfoModIsom, 1, "   extended autos ");

    return NumberOfQuotientsByAllowables( C, G, s );
end );


# Calculates the number of immediate descendants of the given cano form of stepsizes given in the list slist
InstallMethod( NumberOfImmediateDescendantsOfCanoForm, [IsRecord, IsList], function( A, slist )
    local T, G, C, all, i;

    # catch some arguments from  
    T := StructuralCopy(A.cano);
    G := StructuralCopy(A.auto);

    # compute cover
    C := CoveringTable( T );
    Info( InfoModIsom, 1, "   dim(M) = ",C.mul, " and dim(N) = ",C.nuc );
    if C.nuc = 0 then return 0; fi;

    # lift autos
    InduceAutosToMult( G, C, T );
    Info( InfoModIsom, 1, "   extended autos ");

	all := List(slist, x -> NumberOfQuotientsByAllowables( C, G, x ));

    return Sum(all);
end );



# Calculates the number of immediate descendants of the given cano form of all stepsizes
InstallMethod( NumberOfImmediateDescendantsOfCanoForm, [IsRecord], function( A )
    local T, G, C, all, i;

    # catch some arguments from  
    T := StructuralCopy(A.cano);
    G := StructuralCopy(A.auto);

    # compute cover
    C := CoveringTable( T );
    Info( InfoModIsom, 1, "   dim(M) = ",C.mul, " and dim(N) = ",C.nuc );
    if C.nuc = 0 then return 0; fi;

    # lift autos
    InduceAutosToMult( G, C, T );
    Info( InfoModIsom, 1, "   extended autos ");

	all := List([1..C.mul], x -> NumberOfQuotientsByAllowables( C, G, x ));

    return Sum(all);
end );


# Calculates the number of immediate descendants of the given nilpotent associative algebra of stepsize s
InstallMethod( NumberOfImmediateDescendantsOfNAAlg, [IsAlgebra and IsFiniteDimensional, IsPosInt], function( A, s )
    local C;

    C := CanoFormByNAAlg(A);
	return NumberOfImmediateDescendantsOfCanoForm(C, s);
end );


# Calculates the number of immediate descendants of the given nilpotent associative algebra of stepsizes given in the list slist
InstallMethod( NumberOfImmediateDescendantsOfNAAlg, [IsAlgebra and IsFiniteDimensional, IsList], function( A, slist )
    local C;

    C := CanoFormByNAAlg(A);
	return NumberOfImmediateDescendantsOfCanoForm(C, slist);
end );


# Calculates the number of immediate descendants of the given nilpotent associative algebra of all stepsizes
InstallMethod( NumberOfImmediateDescendantsOfNAAlg, [IsAlgebra and IsFiniteDimensional], function( A )
    local C;

    C := CanoFormByNAAlg(A);
	return NumberOfImmediateDescendantsOfCanoForm(C);
end );


InstallMethod( NumberOfNAAlgsByDimAndGens, [IsField and IsFinite, IsPosInt, IsPosInt], function( F, d, n )
    local T, alg, i, j, new, k, A, ret, enum;
    
    T := TrivialCanoForm( F, d );
    alg := List([1..n], x -> []);
    alg[d] := [T];
	ret := 0;

	if d = 1 or d = n then
		return 1;
	fi;

    for i in [d+1..n] do
        for j in [1..Length(alg[i-1])] do
			if i <> n then
           		new := ImmediateDescendantsOfCanoForm( alg[i-1][j], [1..n-i] );
				for k in [1..Length(new)] do
                	Add(alg[new[k].cano.dim], new[k]);
           		od;
			fi;

			#if i = d+1 and IsPrimeField(F) then
			if i = d+1 then
				enum := NumberOfClass2AssociativeAlgebras(d, Size(F), n-d);
			else
				enum := NumberOfImmediateDescendantsOfCanoForm( alg[i-1][j], n-(i-1) );
			fi;

			ret := ret + enum;
        od;
    od;

    return ret;
end );


InstallMethod( NumberOfNAAlgsByDim, [IsField and IsFinite, IsPosInt], function( F, n )
    local algs, d;

    algs := List([1..n], x -> []);
    for d in [1..n] do
        algs[d] := NumberOfNAAlgsByDimAndGens( F, d, n );
    od;

    return Sum(algs);
end );
