# Plots the stepsize 1 descendant tree of a nilpotent associative F-algebra, i.e. the
# descendants of the same coclass.
PlotStepsizeOneDescendantTree := function ( des, P )
    local r, verts, newvert, i, j,  new, k, A, rootdim, str;

	rootdim := des[1][1].cano.dim;
	verts := List( [1..Length(des)], x -> [] );

	for i in [1..Length(des)] do
		for j in [1..Length(des[i])] do 
			# if i = 1, create class for root vertex and plot it
			if i = 1 then
				CreateClass(P, rootdim, 1);
				newvert := Vertex(P, "", rec( levelparam := rootdim, classparam := 1, label := "" ));
				newvert!.cano := des[1][1];
				verts[1] := [newvert];
				CreateClass(P, rootdim + i, j);

			# if these are the last descendants only plot them
			elif i = Length(des) then
				newvert := Vertex(P, "", rec( levelparam := rootdim + i - 1, classparam := des[i][j]!.parent, label := "" ));
				newvert!.cano := des[i][j];
				Add(verts[i], newvert);
				Edge(P, newvert, verts[i-1][des[i][j]!.parent]);

			# in all other cases create classes on next level and plot descendants
			else	
				CreateClass(P, rootdim + i, j);
				newvert := Vertex(P, "", rec( levelparam := rootdim + i - 1, classparam := des[i][j]!.parent, label := "" ));
				newvert!.cano := des[i][j];
				Add(verts[i], newvert);
				Edge(P, newvert, verts[i-1][des[i][j]!.parent]);
			fi;
		od;
	od;
end;


# Plots the coclass graph G_F(r) up to dimension d
InstallMethod( PlotCoclassGraph, [IsField and IsFinite, IsInt, IsPosInt, IsGraphicSheet], function (F, r, d, P)
	local i, roots, des;
	
	Text(P, FONTS.large, 10, 20, JoinStringsWithSeparator(["field:", String(F), "   coclass:", String(r), "   up to dimension:", String(d)], " "));

	# create levels
	for i in [1..d] do
		CreateLevel(P, i, JoinStringsWithSeparator(["dim", i], " "));
		ResizeLevel(P, i, 75);
	od;
	
	# calculate the roots of G_F(r)
	roots := RootsOfCoclassGraph(F, r);
	
	for i in [1..Length(roots)] do
		des := DescendantTreeOfCanoForm(roots[i], d - roots[i].cano.dim);
		PlotStepsizeOneDescendantTree(des, P);
	od;
end );


# Plots the descendant tree of an F-algebra of coclass r up to dimension d
InstallMethod( PlotDescendantTreeOfCanoForm, [IsRecord, IsPosInt, IsGraphicSheet], function (C, d, P)
	local i, des;
	
	Text(P, FONTS.large, 10, 20, JoinStringsWithSeparator(["field:", String(C.cano.fld), "   coclass:", String(CoclassOfCanoForm(C)), "   up to dimension:", String(d)], " "));

	# create levels
	for i in [1..d] do
		CreateLevel(P, i, JoinStringsWithSeparator(["dim", i], " "));
		ResizeLevel(P, i, 75);
	od;

	des := DescendantTreeOfCanoForm(C, d - C.cano.dim);
	PlotStepsizeOneDescendantTree(des, P);
end );


# Returns a list of canonical forms corresponding to the selected vertices in P
InstallMethod( GetSelectedCanoForms, [IsGraphicSheet], function( P )
	local S, i, canos;

	canos := [];
	S := Selected(P);
	for i in [1..Length(S)] do
		Add(canos, S[i]!.cano);
	od;
	return canos;
end );


# Returns a list of algebras corresponding to the selected vertices in P
InstallMethod( GetSelectedNAAlgs, [IsGraphicSheet], function( P )
	local A, i, canos, algs;

	canos := GetSelectedCanoForms(P);
	algs := [];

	for i in [1..Length(canos)] do
		algs[i] := NAAlgByCanoForm(canos[i]);
		if IsBound(canos[i]!.parent) then
			algs[i]!.parent := canos[i]!.parent;
		fi;
	od;

	return algs;
end );
